import dataclasses
import typing

import parso

from odd.artifact import Position
from odd.ast_utils import get_bases, is_op, iter_node_type
from odd.const import UNKNOWN


@dataclasses.dataclass
class ModuleImport:
    start_pos: Position
    end_pos: Position
    from_names: typing.Tuple[str, ...] = dataclasses.field(default_factory=tuple)
    names: typing.Tuple[str, ...] = dataclasses.field(default_factory=tuple)


def get_imports(module) -> typing.Generator[ModuleImport, None, None]:
    for imp in module.iter_imports():
        if imp.type == "import_from":
            from_names = tuple(n.value for n in imp.get_from_names())
        else:
            from_names = ()
        names = tuple(n.value for path in imp.get_paths() for n in path)
        yield ModuleImport(imp.start_pos, imp.end_pos, from_names, names)


def extract_func_name(node: parso.tree.NodeOrLeaf) -> typing.List[str]:
    name_parts = []
    for child in iter_node_type(node, "operator", "name"):
        if is_op(child, "("):
            break
        elif child.type == "name":
            name_parts.append(child.value)
    return name_parts


def get_model_type(classdef_node) -> typing.Union[str, object]:
    bases = get_bases(classdef_node.get_super_arglist())
    for type_, class_ in [
        ("model", "Model"),
        ("transient", "TransientModel"),
        ("abstract", "AbstractModel"),
    ]:
        if (
            (class_,) in bases
            or ("models", class_) in bases
            or ("odoo", "models", class_) in bases
            or ("openerp", "models", class_) in bases
        ):
            return type_
    return UNKNOWN
