from odd.artifact import Confidence, Issue, Location, XMLDocument
from odd.plugin import Plugin


class AttributeOverride(Plugin):
    _handles = {"xml_document"}
    _emits = {"issue"}

    def on_xml_document(self, xml_document: XMLDocument):
        if xml_document.addon.version < 9:
            return
        attributes = ", ".join(f'"{attr}"' for attr in ["class"])
        xpath = "|".join(
            f"/{main_tag}//attribute[@name=({attributes})]"
            for main_tag in ("openerp", "odoo")
        )
        for el in xml_document.node.xpath(xpath):
            is_override = not any(a in el.attrib for a in ("add", "remove"))
            if is_override:
                yield Issue(
                    "attribute_override",
                    f"`<attribute>` overrides the `{el.get('name')}` attribute value, "
                    f'consider using `add="..."` or `remove="..."` instead of '
                    f"overriding",
                    xml_document.addon,
                    locations=[Location(xml_document.path, [el.sourceline])],
                    confidence=Confidence.LOW,
                    categories=["correctness", "maintainability"],
                )
