from odd.artifact import Issue, Location, XMLDocument
from odd.const import SUPPORTED_VERSIONS
from odd.plugin import Plugin
from odd.utils import expand_version_list

ROOT_TAG_VERSION_MAP = expand_version_list(
    {">=8": ["openerp"], ">=9": ["odoo"]}, *SUPPORTED_VERSIONS, result_cls=list
)


class DataElementParent(Plugin):
    _handles = {"xml_document"}
    _emits = {"issue"}

    def on_xml_document(self, xml_document: XMLDocument):
        root_tags = ROOT_TAG_VERSION_MAP[xml_document.addon.version]
        xpath_expr = "//data[%s]" % " and ".join(
            f"not(parent::{root_tag})" for root_tag in root_tags
        )

        for data in xml_document.node.xpath(xpath_expr):
            yield Issue(
                "invalid_data_element_parent",
                f"Expected `<data>` element to be a direct child of one of "
                f"these elements: {', '.join(root_tags)}",
                xml_document.addon,
                locations=[Location(xml_document.path, [data.sourceline])],
                categories=["correctness"],
            )
