from odd.artifact import Addon, Confidence, Issue, Location
from odd.plugin import Plugin
from odd.utils import list_files, lookup_version_list

EXT_VERSION_MAP = {">=8": ["csv", "xml", "sql"], ">=8,<12": ["yml"]}


class DataFileInclusion(Plugin):
    _handles = {"addon"}
    _emits = {"issue"}

    def on_addon(self, addon: Addon):
        extensions = {
            f".{ext}" for ext in lookup_version_list(EXT_VERSION_MAP, addon.version)
        }
        combined_data_files = {*addon.data_files, *addon.demo_files, *addon.qweb_files}
        exclude_dirs = {addon.path / dir for dir in ("tests", "static/src/xml")}

        for file_path in list_files(
            addon.path, list_dirs=False, exclude_dirs=exclude_dirs
        ):
            if file_path.suffix not in extensions:
                continue
            if file_path not in combined_data_files:
                yield Issue(
                    "data_file_missing_in_manifest",
                    "Data file is not included in `demo` or `data` "
                    "sections in the manifest file",
                    addon,
                    locations=[Location(file_path)],
                    confidence=Confidence.LOW,
                    categories=["correctness"],
                )
