import collections
import typing

from odd.artifact import Issue, Location, XMLRecord
from odd.plugin import Plugin

from odd_bunch.xml_utils import collect_fields


class DuplicateRecordFields(Plugin):
    _handles = {"xml_record"}
    _emits = {"issue"}

    def on_xml_record(self, xml_record: XMLRecord):
        record = xml_record.node

        fields: typing.Dict[str, typing.List[int]] = collections.defaultdict(list)
        for field_name, line_no, _, _ in collect_fields(record):
            fields[field_name].append(line_no)

        model, record_id = record.attrib["model"], record.get("id")
        for field_name, line_nos in fields.items():
            if len(line_nos) > 1:
                yield Issue(
                    "duplicate_record_field",
                    f'`{model}` record "{record_id}" has duplicated values '
                    f'for field "{field_name}"'
                    if record_id
                    else (
                        f"`{model}` record has duplicated values "
                        f'for field "{field_name}"'
                    ),
                    xml_record.addon,
                    locations=[
                        Location(xml_record.path, [line_no]) for line_no in line_nos
                    ],
                    categories=["correctness", "maintainability"],
                )
