from odd.artifact import Issue, Location, ModelDefinition
from odd.const import UNKNOWN
from odd.plugin import Plugin
from odd.utils import column_index_1

from odd_bunch.ast_utils import get_model_type
from odd_bunch.utils import odoo_source_url


class NewModelDescription(Plugin):
    _handles = {"model_definition"}
    _emits = {"issue"}

    def on_model_definition(self, model: ModelDefinition):
        if get_model_type(model.node) == UNKNOWN:
            return
        model_name = model.params.get("_name")
        if model.params.get("_inherit") or not model_name:
            return

        if not model.params.get("_description"):
            yield Issue(
                "no_model_description",
                f'Model "{model_name}" has no `_description` set',
                model.addon,
                locations=[
                    Location(model.path, [column_index_1(model.node.start_pos)])
                ],
                categories=(
                    ["future-warning"] if model.addon.version < 12 else ["correctness"]
                ),
                sources=[
                    odoo_source_url(
                        "ff9ddfdacc9581361b555fd5f69e2da61800acad",
                        "odoo/models.py",
                        start=529,
                    )
                ],
            )
