from odd.artifact import AddonPath, Issue, Location
from odd.plugin import Plugin

RECOMMENDED_FILE_PERMISSIONS = 0o644
RECOMMENDED_DIRECTORY_PERMISSIONS = 0o755


class FilePermissions(Plugin):
    _handles = {"addon_path"}
    _emits = {"issue"}

    def on_addon_path(self, addon_path: AddonPath):
        path = addon_path.path
        if path.is_file():
            current_permissions = path.stat().st_mode & 0o777
            if current_permissions != RECOMMENDED_FILE_PERMISSIONS:
                yield Issue(
                    "file_permissions",
                    f"Files should have {RECOMMENDED_FILE_PERMISSIONS:o} "
                    f"permissions (current: {current_permissions:o})",
                    addon_path.addon,
                    locations=[Location(path)],
                    categories=["correctness"],
                )


class DirectoryPermissions(Plugin):
    _handles = {"addon_path"}
    _emits = {"issue"}

    def on_addon_path(self, addon_path: AddonPath):
        path = addon_path.path
        if path.is_dir():
            current_permissions = path.stat().st_mode & 0o777
            if current_permissions != RECOMMENDED_DIRECTORY_PERMISSIONS:
                yield Issue(
                    "directory_permissions",
                    f"Directories should have {RECOMMENDED_DIRECTORY_PERMISSIONS:o} "
                    f"permissions (current: {current_permissions:o})",
                    addon_path.addon,
                    locations=[Location(path)],
                    categories=["correctness"],
                )
