from odd.artifact import Confidence, Issue, Location, Severity, XMLDocument
from odd.plugin import Plugin

XPATH_V8 = "/openerp/data[@noupdate]"
XPATH = "|".join(
    (
        "/data[@noupdate]",
        "/odoo[@noupdate]",
        "/odoo/data[@noupdate]",
        "/openerp[@noupdate]",
        XPATH_V8,
    )
)


class RedundantNoUpdate(Plugin):
    _handles = {"xml_document"}
    _emits = {"issue"}

    def on_xml_document(self, xml_document: XMLDocument):
        """
        Checks for use of `noupdate` attribute in XML data files which are declared
        in `demo` manifest key. `noupdate` has no effect for such records (they will
        be marked as `noupdate` despite the value of the `noupdate` attribute).
        """
        if xml_document.path not in xml_document.addon.demo_files:
            return
        xpath = XPATH_V8 if xml_document.addon.version == 8 else XPATH
        for el in xml_document.node.xpath(xpath):
            yield Issue(
                "redundant_noupdate",
                "`noupdate` attribute on data element `<{tag}>` has no effect for demo "
                "records".format(tag=el.tag),
                xml_document.addon,
                locations=[Location(xml_document.path, [el.sourceline])],
                severity=Severity.INFO,
                confidence=Confidence.HIGH,
                categories=["redundancy"],
            )
