from odd.plugin import Plugin
from odd.artifact import Confidence, Issue, Location, XMLDocument


class RedundantTAttf(Plugin):
    _handles = {"xml_document"}
    _emits = {"issue"}

    def on_xml_document(self, xml_document: XMLDocument):
        for el in xml_document.node.xpath(
            "//template//*/@*[starts-with(name(), 't-attf-')]/.."
        ):
            for name, value in el.attrib.items():
                if not name.startswith("t-attf-"):
                    continue

                is_format_string = ("{{" in value and "}}" in value) or (
                    "#{" in value and "}" in value
                )
                if not is_format_string:
                    yield Issue(
                        "redundant_t_attf",
                        f"Element `<{el.tag}>` has a redundant `t-attf-$name` "
                        f"attribute `{name}`: {value}",
                        xml_document.addon,
                        locations=[Location(xml_document.path, [el.sourceline])],
                        # Might be a valid case if a static string is used
                        # inside `t-attf-*` in an original template, only to allow
                        # an extension to replace it with a dynamic value, hence the
                        # low confidence.
                        confidence=Confidence.LOW,
                        # FIXME: Should be ["redundancy"].
                        categories=["correctness", "performance"],
                    )
