from odd.artifact import Issue, Location, PythonModule
from odd.ast_utils import iter_node_type
from odd.const import SUPPORTED_VERSIONS
from odd.plugin import Plugin
from odd.utils import column_index_1, expand_version_list

from odd_bunch.ast_utils import extract_func_name

ROUTE_KWARG_VERSION_MAP = {
    ">=8": ["auth", "methods", "multilang", "type", "website"],
    ">=9": ["cors", "csrf"],
    ">=11": ["sitemap"],
    ">=12": ["save_session"],
}

ROUTE_KWARGS = expand_version_list(
    ROUTE_KWARG_VERSION_MAP, *SUPPORTED_VERSIONS, result_cls=set
)


class RouteKwargs(Plugin):
    _handles = {"python_module"}
    _emits = {"issue"}

    def on_python_module(self, python_module: PythonModule):
        for node in iter_node_type(python_module.node, "decorator"):
            name_parts = extract_func_name(node)
            if (len(name_parts) == 1 and name_parts[0] != "route") or (
                len(name_parts) == 2
                and (name_parts[0] != "http" or name_parts[1] != "route")
            ):
                continue

            kwargs = {
                c.children[0].value: (
                    c.children[0].line,
                    column_index_1(c.children[0].start_pos),
                )
                for c in iter_node_type(node, "argument")
            }

            for kw in kwargs.keys() - ROUTE_KWARGS[python_module.addon.version]:
                yield Issue(
                    "unknown_route_kwarg",
                    f'Unknown `http.route()` keyword argument "{kw}"',
                    python_module.addon,
                    locations=[Location(python_module.path, [kwargs[kw][0]])],
                    categories=["correctness"],
                )
