import typing

from odd.artifact import Issue, Location, PythonModule
from odd.ast_utils import get_bases
from odd.plugin import Plugin
from odd.utils import column_index_1

from odd_bunch.ast_utils import extract_func_name, get_imports
from odd_bunch.utils import odoo_commit_url


class UnitTestTestCaseTagged(Plugin):
    _handles = {"python_module"}
    _emits = {"issue"}

    def on_python_module(self, python_module: PythonModule):
        if python_module.addon.version < 12 or not python_module.path.name.startswith(
            "test_"
        ):
            return

        imports: typing.Set[typing.Union[typing.Tuple[str, ...], str]] = set()
        for imp in get_imports(python_module.node):
            from_part = ".".join(imp.from_names)
            if from_part:
                imports.update((from_part, n) for n in imp.names)
            else:
                imports.update(imp.names)

        for classdef in python_module.node.iter_classdefs():
            bases = get_bases(classdef.get_super_arglist())
            is_test_case = (
                ("unittest", "TestCase") in bases and "unittest" in imports
            ) or (("TestCase",) in bases and ("unittest", "TestCase") in imports)
            if not is_test_case:
                continue

            tagged = False
            for decorator in classdef.get_decorators():
                if extract_func_name(decorator)[-1] == "tagged":
                    tagged = True
                    break
            if not tagged:
                yield Issue(
                    "unittest_testcase_not_tagged",
                    f"`unittest.TestCase` subclass `{classdef.name.value}` is not "
                    f"decorated with `@tagged()`, it will not be picked up by Odoo "
                    f"test runner",
                    python_module.addon,
                    locations=[
                        Location(
                            python_module.path, [column_index_1(classdef.start_pos)]
                        )
                    ],
                    categories=["correctness"],
                    sources=[
                        odoo_commit_url("b356b190338e3ee032b9e3a7f670f76468965006")
                    ],
                )
