from odd.artifact import Issue, Location, XMLDocument
from odd.plugin import Plugin

ELEMENTS = {"record", "template", "act_window", "menuitem", "report"}
XPATH_EXPR = "|".join(f"//{el}[not(@id)]" for el in ELEMENTS)


class XMLOperationNoID(Plugin):
    _handles = {"xml_document"}
    _emits = {"issue"}

    def on_xml_document(self, xml_document: XMLDocument):
        for el in xml_document.node.xpath(XPATH_EXPR):
            yield Issue(
                "xml_operation_without_id",
                f"XML operation `<{el.tag}>` has no `id` attribute",
                xml_document.addon,
                locations=[Location(xml_document.path, [el.sourceline])],
                categories=["maintainability"],
            )
