from typing import Optional

import yarl


def odoo_commit_url(commit: str) -> str:
    return f"https://github.com/odoo/odoo/commit/{commit}"


def odoo_reference_url(version: int, page: str, *, fragment="") -> str:
    if page not in (
        "orm",
        "data",
        "actions",
        "views",
        "module",
        "cmdline",
        "security",
        "testing",
        "http",
        "qweb",
        "javascript_cheatsheet",
        "javascript_reference",
        "translations",
        "reports",
        "mixins",
        "guidelines",
        "mobile",
    ):
        raise ValueError(f"Unknown reference page: {page:s}")
    return str(
        yarl.URL.build(
            scheme="https",
            host="www.odoo.com",
            path=f"/documentation/{version:d}.0/reference/{page:s}.html",
            fragment=fragment,
        )
    )


def odoo_source_url(
    commit: str, path: str, *, start: Optional[int] = None, end: Optional[int] = None
) -> str:
    if end and not start:
        raise ValueError("`start` must be provided if `end` is provided")

    fragment = ""
    if start is not None:
        fragment = f"L{start:d}" if end is None else f"L{start:d}-L{end:d}"

    return str(
        yarl.URL.build(
            scheme="https",
            host="github.com",
            path=f"/odoo/odoo/blob/{commit}/{path}",
            fragment=fragment,
        )
    )
