import pathlib
import re
import typing

import lxml.etree as ET  # nosec


def get_xpath_expr_target_element(
    xpath_expr: typing.Optional[str]
) -> typing.Optional[str]:
    if not xpath_expr:
        return None
    last_part = xpath_expr.split("/")[-1]
    m = re.search(r"(?P<modifier>[^:]+::)?(?<!@)(?P<nodename>[^\[]+)", last_part)
    nodename = None
    if m is not None:
        nodename = m.group("nodename")
    if nodename == ".":
        nodename = None
    return nodename


def drop_comments(iterable):
    for node in iterable:
        if node.tag is not ET.Comment:
            yield node


def get_root(path: pathlib.Path):
    with path.open(mode="rb") as f:
        return ET.parse(f).getroot()


def get_view_arch(record):
    for field in record.iterchildren(tag="field"):
        if field.get("name") == "arch":
            return field
    return None


def collect_fields(
    record
) -> typing.Generator[
    typing.Tuple[str, int, typing.Dict[str, str], typing.Optional[str]], None, None
]:
    for field in record.iterchildren(tag="field"):
        attribs = dict(field.attrib)
        yield attribs["name"], field.sourceline, attribs, field.text
